require 'dockerploy'
require 'thor'
require 'yaml'
require 'dockerploy/configuration'
require 'dockerploy/deploy'
require 'dockerploy/image'
require 'dockerploy/template'
require 'erb'
require 'ostruct'

module Dockerploy
  # The command-line interface for Dockerploy
  class CLI < Thor
    class_option :config, aliases: ['-c'], type: :string, required: false
    class_option :tag, aliases: ['-t'], type: :string, required: false

    def initialize(args = [], opts = {}, config = {})
      super(args, opts, config)
    end

    desc 'init', 'Generate a config.yml file'
    def init
      Template.write
    end

    desc 'ps ENVIRONMENT', 'Show running containers'
    def ps(environment)
      config = configuration(environment.to_sym)
      return unless config.servers
      config.servers.each do |server|
        ssh_client = SSHClient.new(server[:host], server[:username], server[:password], server[:port])
        ssh_client.command(sprintf('docker ps | grep %s', config.application_name))
      end
    end

    desc 'build', 'build an image'
    def build
      build_options = { tag: options[:tag] }
      Image.new(configuration, build_options).build
    end

    desc 'push', 'push an image'
    def push
      build_options = { tag: options[:tag] }
      Image.new(configuration, build_options).push
    end

    desc 'pull', 'pull an image'
    def pull(environment)
      build_options = { tag: options[:tag] }
      Image.new(configuration(environment.to_sym), build_options).pull
    end

    desc 'deploy', 'deploy an application'
    def deploy(environment)
      deploy_options = { tag: options[:tag] }
      config = configuration(environment.to_sym)
      Deploy.new(config, deploy_options).deploy
    end

    private

    def configuration(env = nil)
      @configuration ||= Configuration.new(env: env)
    end
  end
end
