module Dockerploy
  class Configuration
    NoEnvError = Class.new(StandardError)
    FileNotFound = Class.new(StandardError)
    DEFAULT_CONFIG_FILE = 'docker/config.yml'.freeze

    attr_reader :env, :application_name, :docker_host, :image_name

    def initialize(options = {})
      @config_file = options.fetch(:config_file, DEFAULT_CONFIG_FILE)
      @env = options.fetch(:env)
      @env = @env.to_sym if @env
      raise FileNotFound unless File.exist?(@config_file)
      config = YAML.load_file(@config_file).symbolize_keys
      @application_name = config.fetch(:application_name)
      @docker_host = config.fetch(:docker_host)
      @image_name = config.fetch(:image_name)
    end

    def servers
      raise NoEnvError if @env.nil?
      options[@env][:servers]
    end

    def volumes
      raise NoEnvError if @env.nil?
      options[@env][:volumes]
    end

    def env_file
      raise NoEnvError if @env.nil?
      options[@env][:env_file]
    end

    def branch
      raise NoEnvError if @env.nil?
      options[@env][:branch]
    end

    def tagging?
      raise NoEnvError if @env.nil?
      options[@env][:tagging]
    end

    def image_tag
      raise NoEnvError if @env.nil?
      options[@env][:image_tag]
    end

    def before_all_hooks
      return {} if options[:hooks].nil? || options[:hooks][:before_deploy].nil?
      options[:hooks][:before_deploy]
    end

    def before_hooks
      raise NoEnvError if @env.nil?
      return {} if options[@env].nil?
      hooks = options[@env][:hooks]
      return {} if hooks.nil? || hooks[:before_deploy].nil?
      hooks[:before_deploy]
    end

    def after_all_hooks
      return {} if options[:hooks].nil? || options[:hooks][:after_deploy].nil?
      options[:hooks][:after_deploy]
    end

    def after_hooks
      raise NoEnvError if @env.nil?
      return {} if options[@env].nil?
      hooks = options[@env][:hooks]
      return {} if hooks.nil? || hooks[:after_deploy].nil?
      hooks[:after_deploy]
    end

    private

    def options
      @options ||= begin
        raise FileNotFound unless File.exist?(@config_file)
        YAML.load_file(@config_file).symbolize_keys
      end
    end
  end
end